package unitTest;

import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DynamicTest;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestFactory;
import org.junit.jupiter.api.function.Executable;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;

import exceptions.CategoryInvalidException;
import exceptions.DateInvalidException;
import exceptions.DescriptionInvalidException;
import exceptions.ImageInvalidException;
import exceptions.ItemNonExistentException;
import exceptions.PriceInvalidException;
import exceptions.TitleInvalidException;
import project.Auction;
import project.Category;
import project.Item;
import project.ItemPicture;
import project.Seller;
import project.User;

public class SellerUnitTest {

	@BeforeEach
	public void reset() {

		Item.items = new ArrayList<Item>();
		User.users = new ArrayList<User>();
		Auction.auctions = new ArrayList<Auction>();
		Category.categories = new ArrayList<Category>();

	}

	@ParameterizedTest
	@ValueSource(ints = { 0, -1, 3, 5 })
	public void createAuctionError(int itemId)
			throws PriceInvalidException, DateInvalidException, ItemNonExistentException {

		Seller s = Seller.getInstance();
		assertThrows(ItemNonExistentException.class,
				() -> s.createAuction(Item.getItem(itemId), "22/10/2019 11:20:13", "23/10/2019 11:20:13", 10, 5));

	}

	@ParameterizedTest
	@ValueSource(ints = { 0, -1, 3, 5 })
	public void createAuctionWorking(int itemId)
			throws PriceInvalidException, DateInvalidException, ItemNonExistentException, ParseException {

		Item i = new Item();
		i.setId(itemId);

		Seller s = Seller.getInstance();
		assertTrue(s.createAuction(Item.getItem(itemId), "22/10/2019 11:20:13", "23/10/2019 11:20:13", 10, 5));

	}

	@TestFactory
	public Collection<DynamicTest> createAuctionAcceptanceTest() {

		Collection<DynamicTest> dynamicTests = new ArrayList<DynamicTest>();

		Seller s = Seller.getInstance();
		ArrayList<Integer> items = new ArrayList<Integer>(Arrays.asList(0));

		Item item = new Item();
		item.setId(1);

		for (int i = 0; i < items.size(); i++) {

			final int j = i;

			Executable exec = () -> assertThrows(ItemNonExistentException.class, () -> s
					.createAuction(Item.getItem(items.get(j)), "11/12/2019 11:50:11", "12/12/2019 12:50:11", 10d, 5d));
			DynamicTest dTest = DynamicTest.dynamicTest("Item not existent", exec);

			dynamicTests.add(dTest);

		}

		ArrayList<String> startTimes = new ArrayList<String>(
				Arrays.asList("11/11/2019 11:20:12", "111112019", "10/11/2019", "111112019"));
		ArrayList<String> endTimes = new ArrayList<String>(
				Arrays.asList("10/11/2019 11:20:12", "10112019", "111112019", "11/11/2019"));
		ArrayList<String> testTimeNames = new ArrayList<String>(Arrays.asList("End time before Start time",
				"Both dates in a wrong format", "End time in a wrong format", "Start time in a wrong format"));

		for (int i = 0; i < startTimes.size(); i++) {

			final int j = i;

			Executable exec = () -> assertThrows(DateInvalidException.class,
					() -> s.createAuction(Item.getItem(1), startTimes.get(j), endTimes.get(j), 10d, 5d));
			DynamicTest dTest = DynamicTest.dynamicTest(testTimeNames.get(j), exec);

			dynamicTests.add(dTest);

		}

		ArrayList<Integer> prices = new ArrayList<Integer>(Arrays.asList(-1, 0));
		ArrayList<String> testPriceNames = new ArrayList<String>(
				Arrays.asList("Negative price", "Price equal to zero"));

		for (int i = 0; i < prices.size(); i++) {

			final int j = i;

			Executable exec = () -> assertThrows(PriceInvalidException.class, () -> s.createAuction(Item.getItem(1),
					"11/12/2019 21:50:11", "12/12/2019 21:50:11", prices.get(j), 5d));
			DynamicTest dTest = DynamicTest.dynamicTest(testPriceNames.get(j), exec);

			dynamicTests.add(dTest);

		}

		ArrayList<Integer> bidIncrements = new ArrayList<Integer>(Arrays.asList(-1, 0));
		ArrayList<String> testIncrementNames = new ArrayList<String>(
				Arrays.asList("Negative increment", "Increment equal to zero"));

		for (int i = 0; i < bidIncrements.size(); i++) {

			final int j = i;

			Executable exec = () -> assertThrows(PriceInvalidException.class, () -> s.createAuction(Item.getItem(1),
					"11/12/2019 21:50:11", "12/12/2019 21:50:11", 10, bidIncrements.get(j)));
			DynamicTest dTest = DynamicTest.dynamicTest(testIncrementNames.get(j), exec);

			dynamicTests.add(dTest);

		}

		return dynamicTests;

	}

	@TestFactory
	public Collection<DynamicTest> createItemAcceptanceTest() {

		Collection<DynamicTest> dynamicTests = new ArrayList<DynamicTest>();

		Seller s = Seller.getInstance();

		new Category("Sport and Outdoor");

		ItemPicture ip1 = new ItemPicture();
		ItemPicture ip2 = new ItemPicture();
		ip1.setId(1);
		ip2.setId(2);

		ArrayList<String> titles = new ArrayList<String>(Arrays.asList("", "9999", "t-#\\", "too long"));
		ArrayList<String> testTitleNames = new ArrayList<String>(
				Arrays.asList("Title empty", "Numeric title", "Symbols in the title", "Title too long"));

		for (int i = 0; i < titles.size(); i++) {

			String prepareTitle = titles.get(i);

			if (i == 3) {

				for (int k = 0; k < 60; k++) {

					prepareTitle += "a";

				}
			}

			final String title = prepareTitle;
			final int j = i;

			Executable exec = () -> assertThrows(TitleInvalidException.class,
					() -> s.createItem(title, "Beautiful bike", new int[] { 1, 2 }, "Sport and Outdoor"));
			DynamicTest dTest = DynamicTest.dynamicTest(testTitleNames.get(j), exec);

			dynamicTests.add(dTest);

		}

		ArrayList<String> descriptions = new ArrayList<String>(Arrays.asList("", "9999", "too long"));
		ArrayList<String> testDescriptionNames = new ArrayList<String>(
				Arrays.asList("Description empty", "Numeric description", "Description too long"));

		for (int i = 0; i < descriptions.size(); i++) {

			String prepareDescription = descriptions.get(i);

			if (i == 2) {

				for (int k = 0; k < 600; k++) {

					prepareDescription += "a";

				}
			}

			final String description = prepareDescription;
			final int j = i;

			Executable exec = () -> assertThrows(DescriptionInvalidException.class,
					() -> s.createItem("Red bike", description, new int[] { 1, 2 }, "Sport and Outdoor"));
			DynamicTest dTest = DynamicTest.dynamicTest(testDescriptionNames.get(j), exec);

			dynamicTests.add(dTest);

		}

		ArrayList<String> testItemPictureNames = new ArrayList<String>(
				Arrays.asList("No picture inserted", "Too many pictures"));

		for (int i = 0; i < 2; i++) {

			int[] prepareImages = { 1, 2 };

			if (i == 0) {

				prepareImages = new int[0];

			} else if (i == 1) {

				prepareImages = new int[11];

				for (int l = 0; l < 11; l++) {

					ItemPicture ipn = new ItemPicture();
					ipn.setId(l + 10);

					prepareImages[l] = l + 10;

				}

			}

			final int[] images = prepareImages;

			Executable exec = () -> assertThrows(ImageInvalidException.class,
					() -> s.createItem("Red bike", "Beautiful bike", images, "Sport and Outdoor"));
			DynamicTest dTest = DynamicTest.dynamicTest(testItemPictureNames.get(i), exec);

			dynamicTests.add(dTest);

		}

		ArrayList<String> categories = new ArrayList<String>(Arrays.asList(""));

		for (int i = 0; i < categories.size(); i++) {

			final int j = i;

			Executable exec = () -> assertThrows(CategoryInvalidException.class,
					() -> s.createItem("Red bike", "Beautiful Bike", new int[] { 1, 2 }, categories.get(j)));
			DynamicTest dTest = DynamicTest.dynamicTest("Category not existent", exec);

			dynamicTests.add(dTest);

		}

		return dynamicTests;

	}

	@Test
	public void createItemWorking()
			throws CategoryInvalidException, TitleInvalidException, DescriptionInvalidException, ImageInvalidException {

		Seller s = Seller.getInstance();

		ItemPicture ip1 = new ItemPicture();
		ItemPicture ip2 = new ItemPicture();
		ip1.setId(1);
		ip2.setId(2);

		new Category("Sport and Outdoor");

		int[] images = { 1, 2 };

		assertTrue(s.createItem("Red Bike", "Beautiful red bike, contact me", images, "Sport and Outdoor"));

	}

}
